%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Copyright (C) 2004 Adam Spiro and Aharon Bar-Hillel
%    An implementation of a spike sorting algorithm according to the paper:
%    "Spike Sorting: Bayesian Clustering of Non-Stationary Data" 
%    by Aharon Bar-Hillel, Adam Spiro and Eran Stark 
%    (http://www.cs.huji.ac.il/~aharonbh/papers/SpikeSort_NIPS04.ps)
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%
% path_human_flag: 0 - Remix the solutions with the chosen path, i.e, take
%                     the solutions of the chosen path and mix them with
%                     adaptations to the frames.
%                  1 - Take the Human's Solution (indicated 'tag') and
%                      Transfer them as is (without adaptations).
%                  2 - Take the Human's Solution (indicated 'tag') and
%                      Remix them with adaptations.
%
%

function [Ppath,PSolutions, PLL,PBiMixScores , PCorrespondences,PLabels,  PRecordGraph]=...
            RemixWithPath(Solutions,LL,path,data,time,GParams,N,tag,path_human_flag)


        
%SetAllParams;

% The first mixing was TransferedSolNum=1,TimeOffset=5,MixingRepeatition=2

% GParams.TransferedSolNum=3;         % the number of solotions one tries to transfer
%                                                                      % from one time frame to the other.
% GParams.TimeOffset=6;                     % the time distance (in number of frames) from which
%                                                                       %  solutions are sought for transfer.
% GParams.MixingRepeatition=3;        % Number of times to call the mixingSolutions function
% 
% GParams.FramesFromHumanInterval=5;  % The interval of frames from which to take solutions from the human

if (~exist('path_human_flag'))
    path_human_flag=0;
end

if (path_human_flag==0) % remix with the chosen path
	PSolutions=cell(size(Solutions));
	PLL=zeros(size(LL));
	for i=1:size(path,2)
        PSolutions(i,1)=Solutions(i,path(i));
        if (isempty(PSolutions{1,1}{end,4}))
            tmp=PSolutions{i,1}(1,:);
            PSolutions{i,1}(1,:)=PSolutions{i,1}(end,:);
            PSolutions{i,1}(end,:)=tmp;
        end
        PLL(i,1)=LL(i,path(i));
	end
else % remix only with the human's tag
    PSolutions=cell(size(Solutions,1),1);
    PLL=ones(size(LL,1),1)*(-inf);
    % take every GParams.FramesFromHumanInterval frames
    FramesSetByHuman=[1 (1:floor(size(Solutions,1)/GParams.FramesFromHumanInterval))*GParams.FramesFromHumanInterval]; 
	for frame=FramesSetByHuman
        Inds=sum(N(1:frame-1))+1:sum(N(1:frame));
        H_Labels=tag(Inds); % the human labels of the currnt frame.
        tData=data(Inds,:); % the frame's data points.
        
        s=1;
        Sol=cell(size(unique(H_Labels),2),4);
        for label=unique(H_Labels)
            Lind = find(H_Labels==label);
            Mean=sum(tData(Lind,:))/size(tData(Lind,:),1);
            M=[tData(Lind,1)-Mean(1) tData(Lind,2)-Mean(2)];
            Var=M'*M/(size(tData(Lind,:),1)-1);
            alpha=sum(H_Labels==label)/size(H_Labels,2);
            Sol{s,1}=Mean;
            Sol{s,2}=Var;
            Sol{s,3}=alpha;
            if label==1 % the background
                Sol{s,4}=[];
            else
                Sol{s,4}=zeros(1,size(unique(H_Labels),2));
            end
            s=s+1;
        end
        
        bg=find(cellfun('isempty',Sol(:,4))==1); % the background model index
        tmpK=size(Sol,1);
        Sol=Sol([bg setdiff(1:tmpK,bg)],:);
        
        % Update Solutions
        PSolutions{frame,1}=Sol;
        
        % Update LL
        [stam , logLikelihood ]=calc_map_labeling2(tData,Sol);
        PLL(frame,1) = logLikelihood;
    end
    HSolutions=PSolutions;
    HLL=PLL;
   % Now HSolutions/HLL contains only the humans labelings.
end



if (path_human_flag==1) % Transfer the human's solutions as is (without adaptations)
	c=1;
	for i=1:size(HLL,1)
        if (isempty(HSolutions{i,1}))
            a1=i-mod(i,GParams.FramesFromHumanInterval);
            a2=a1+GParams.FramesFromHumanInterval;
            if (a2>size(HLL,1))
                a2=a1-GParams.FramesFromHumanInterval;
            end
            if (a1<=0)
                a1=1;
            end
            
            PSolutions{i,1}=HSolutions{a1,1};
            [stam , PLL(i,1) ]=calc_map_labeling2(data(c:(c+N(i)-1),:),PSolutions{i,1});
            PSolutions{i,2}=HSolutions{a2,1};
            [stam , PLL(i,2) ]=calc_map_labeling2(data(c:(c+N(i)-1),:),PSolutions{i,1});
            
            
            Inds=sum(N(1:i-1))+1:sum(N(1:i));
            tData=data(Inds,:); % the frame's data points.
            chunks=-ones(N(i),1);
            K = size(PSolutions{i,1},1)-1;
            GParams.EMParams.param=PSolutions{i,1};
            GParams.EMParams.num_iterations=1;
            [ PSolutions{i,3}, tmpLL, Groups, z, stam ,exit_flags ]=...
                mixture_of_gaussians5(tData, K, chunks, [],GParams.EMParams);
            PLL(i,3)=tmpLL(end);    
            
            K = size(PSolutions{i,2},1)-1;
            GParams.EMParams.param=PSolutions{i,2};
            GParams.EMParams.num_iterations=1;
            [ PSolutions{i,4}, tmpLL, Groups, z, stam ,exit_flags ]=...
                mixture_of_gaussians5(tData, K, chunks, [],GParams.EMParams);
            PLL(i,4)=tmpLL(end);               
        end
        c=c+N(i);
	end
	PLL(FramesSetByHuman,2)=-inf;
    [PSolutions, PLL]=PruneAndSort(PSolutions, PLL,GParams,data,N,0,1,1);
else
	%Remix the solutions with local adaptations
	for repeat=1:GParams.MixingRepeatition
        disp(sprintf('Mixing Solutions iteration %d',repeat));
        Ssize = size(PSolutions,2);
        [PSolutions, PLL] = mixingSolutions(PSolutions,PLL,data,time,GParams,N);
        [PSolutions, PLL]=PruneAndSort(PSolutions, PLL,GParams,data,N,1,1,Ssize+1); % prune the new solutions and sort
	end
end

if (path_human_flag~=0)
	% now replace the humans solutions to be the only one
	for i=1:size(PSolutions,1)
        if (~isempty(HSolutions{i,1}))
            PSolutions(i,:)=cell(1,size(PSolutions,2));
            PSolutions{i,1}=HSolutions{i,1};
            PLL(i,:)=ones(1,size(PSolutions,2))*(-inf);
            PLL(i,1)=HLL(i,1);
        end
	end
end
    
[PSolutions, PLL]=PruneAndSort(PSolutions, PLL,GParams,data,N,0,1,1); % Now only sort without prun

PSolutions=reorderBackGround(PSolutions);

[ PBiMixScores , PCorrespondences , PZ] = calc_mix_scores(PSolutions,N,GParams);


Ppath = calc_mix_Viterbi(PLL,PBiMixScores,GParams);

[ PLabels,  PRecordGraph ]=prepareUnitedLabels(data,PSolutions,N,Ppath,PCorrespondences,GParams);
