%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Copyright (C) 2004 Adam Spiro and Aharon Bar-Hillel
%    An implementation of a spike sorting algorithm according to the paper:
%    "Spike Sorting: Bayesian Clustering of Non-Stationary Data" 
%    by Aharon Bar-Hillel, Adam Spiro and Eran Stark 
%    (http://www.cs.huji.ac.il/~aharonbh/papers/SpikeSort_NIPS04.ps)
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% set all params script. Used in spike sorting main3

GParams.DataDim=2;

GParams.N=1000;
GParams.DataInd=1;

% Gathering solution stage parameters:

% Window  sizs and numbers
GParams.UseConstWindowSizeFlag=1;
[N1 D1]=size(data);
GParams.ExpectedSpikeInWindow=1000; % data is seperated into windows with expected spike count of ExpectedSpikeInWindow
if GParams.UseConstWindowSizeFlag
    siw=GParams.N(GParams.DataInd);
    GParams.WinNum=floor(length(data)/siw);
    GParams.SIW=ones(1,GParams.WinNum)*siw;     % number of spikes in all the window
    GParams.SIW(end)=length(data)-(GParams.WinNum-1)*siw;
else
    GParams.WinNum=floor(N1/GParams.ExpectedSpikeInWindow);      
    GParams.TimeWin=time(end)/GParams.WinNum;      % time length of a window
end


% Basic Solutions parameters
GParams.SolNum=16;      % number of GMM clusterings gathered at each time window.
GParams.minK=1;                % minimum number of clusters to look for (Fixed for now)
GParams.maxK=6;                % maximum number of clusters to look for (Fixed for now)


% Mixing solutions parameters
GParams.TransferedSolNum=3;         % the number of solotions one tries to transfer
                                                                     % from one time frame to the other.
GParams.TimeOffset=6;                     % the time distance (in number of frames) from which
                                                                      %  solutions are sought for transfer.
GParams.MixingRepeatition=3;        % Number of times to call the mixingSolutions function

GParams.FramesFromHumanInterval=5;  % The interval of frames from which to take solutions from the human

% pre and post processing parameters
GParams.SmallClusterThreshold=1; % If a cluster is smaller then this value (in percentage) move it to the background
         
GParams.ConfidenceMin=0;    % clustering confidence required for a point to be labeled.
GParams.MaxNumOfColors = 20; % the maximum number of colors to use


% EM parameters :
GParams.EMParams.BgrModelZeroMean=1; % set the center of the backgroung model to 0.
GParams.BrgStrength=40;  % Beckground model strength.
GParams.EMParams.single_cov_mat_flag=0;
GParams.EMParams.BgrModel=GParams.BrgStrength;
GParams.EMParams.number_of_starts=4;
GParams.EMParams.MaxLLFlag=1;


% prunning solution threshold
GParams.PrunDist=0.03;
GParams.MaxBGSize=0.05;

% mixture distance parameters
GParams.BGF=1;      % Background first flag : when 1, Backgrounds are united first
GParams.MethodType=1;   % when 1 : lose at least one bit in labels, encode only Gaussain offsets
                                                  % When 2 :  lose 'exactly' one bit in labels, encode only Gaussian offsets
                                                  % 3 :Not supported yet.
GParams.SafConst=1;         % currently supported only for Methodtype 1. The number of 
                                                   % bits required to be saved in the joint mixture
GParams.SafBreakdown=0.4;       % The number of bits required to be saved in a breakdown state.
                                                            % only supported for  MethodType=4                                          
GParams.BreakdownBitCost=1;     % the cost of the additional bits when a breakdown state is choosen.


%%%%%%%%%%%%%%%%%%%%%%%%
% Prunning values
%%%%%%%%%%%%%%%%%%%%%%%%
GParams.ValleyHeight = 0.1; % The height of the valley between two peaks in which we want to exclude
GParams.GaussianLikeScore=1500; % The score of the cluster which measures the similarity to a gaussian distribution

%%%%%%%%%%%%%%%%%%%%%%%%
% Post processing values
%%%%%%%%%%%%%%%%%%%%%%%%
GParams.SizeOfSmallClusterWithLargeCov=0.02; % the size of the small clusters which have a big covariance we want to remove
GParams.RareClusterRemoval=3000; % Number of spikes for which smaller clusters are removed

%%%%%%%%%%%%%%%%%%%%%%%%
% Viterbi parameters
%%%%%%%%%%%%%%%%%%%%%%%%
GParams.ContinuityParam=1; % How much does the solution has to be a continuity one (e.g =1000).

%%%%%%%%%%%%%%%%%%%%%%%%
% Unstationary Thresholds (see check_unstat.m)
%%%%%%%%%%%%%%%%%%%%%%%%
GParams.SingleTransitionTH=-300;
GParams.AverageTransitionTH=-120;


% debugging flags
GParams.FixedSeedFlag=1;
GParams.FixedSeed=2345;


% visual output
plot_type = 1; % plot_type==1  ===>  Plots only the solution we found.
               % plot_type==2  ===>  Plots the divided table also with Eran's solution and
               % the pos tags.

