%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Copyright (C) 2004 Adam Spiro and Aharon Bar-Hillel
%    An implementation of a spike sorting algorithm according to the paper:
%    "Spike Sorting: Bayesian Clustering of Non-Stationary Data" 
%    by Aharon Bar-Hillel, Adam Spiro and Eran Stark 
%    (http://www.cs.huji.ac.il/~aharonbh/papers/SpikeSort_NIPS04.ps)
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% The score is the JS of the most seperable cluster:
% This is a MaxMin search - For each cluster we calculate the score of the nearest cluster (Min).
% Then, we return the score of the maximal JS score(Max).
function [score,C] = calcSeperateScore_JS(Sol,N)

%preapare the vars
M1=Sol;

tmp = size(M1);
BlueK=tmp(1);

totN=N(1);

% prepare a table 2*(number of Gaussians) indexed by the Gaussian number, 
% which says how many blue and red Gaussians  are part of the Gaussian.
Color=zeros(BlueK,2);       
Color(1:BlueK,1)=1;

% create the starting pool of Gaussians. 
Gauss=cell(BlueK,5);     % a cell containing the mixture.
Gauss(1:BlueK,1:3)=M1(1:BlueK,1:3);

for i=1:BlueK
    Gauss{i,3}=Gauss{i,3}*(Color(i,:)*N') / totN;
    Gauss{i,4}=det(Gauss{i,2});
    Gauss{i,5}=i;       % the initial gaussians which are part of the Gaussian.
end

% calculate the JS between each cluster
for i1=2:size(Sol,1)
    for i2=2:size(Sol,1)
        s(i1-1,i2-1)=g_dist3(Gauss(i1,:),Gauss(i2,:));
    end
end
s(find(s==0))=inf;

% return the score of the most seperable cluster.
score=max(min(s'));

[a b]=find(s==score);
C=[a(1)+1 b(1)+1];