%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Copyright (C) 2004 Adam Spiro and Aharon Bar-Hillel
%    An implementation of a spike sorting algorithm according to the paper:
%    "Spike Sorting: Bayesian Clustering of Non-Stationary Data" 
%    by Aharon Bar-Hillel, Adam Spiro and Eran Stark 
%    (http://www.cs.huji.ac.il/~aharonbh/papers/SpikeSort_NIPS04.ps)
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% computes the assignments and LL of a data given a MOG.
% the function returns :
%   mapAssignment - the assignment for each data point
%   CodeLL - the coding cost of all the points : the label cost+the gaussian offset cost.

function [ mapAssignment, CodeLL ]= calc_map_labeling2(data,param)

[rows cols]= size(data);
n= rows;
d= cols;

M= (size(param,1));
singleCovMatFlag=0;

if size(param,1)>1 && isempty(param{2,1})
    singleCovMatFlag=1;
end

for j=1:M
   if (singleCovMatFlag == 1)
      cov_index=1;
   else
      cov_index=j;
   end
   
   tempData=data-ones(n,1)*param{j,1}; % subtract the mean
   q1=tempData*inv(param{cov_index,2}).*tempData;
   expContent=-0.5*sum(q1,2);         % the exponent content

   LogProbs(:,j)= -(d/2)*log(2*pi) -0.5*log(det(param{cov_index,2}))+expContent;
   LogProbs(:,j)=LogProbs(:,j)+log(param{j,3});

end

m=max(LogProbs');
[  stam mapAssignment ]=max(LogProbs');
CodeLL=sum(m);

