%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Copyright (C) 2004 Adam Spiro and Aharon Bar-Hillel
%    An implementation of a spike sorting algorithm according to the paper:
%    "Spike Sorting: Bayesian Clustering of Non-Stationary Data" 
%    by Aharon Bar-Hillel, Adam Spiro and Eran Stark 
%    (http://www.cs.huji.ac.il/~aharonbh/papers/SpikeSort_NIPS04.ps)
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Compute the Maximum-A-Posteriory path of clusterings.
% input :  logA - the transition probablities
%                 logB - the emision probabilities
% output: path : a vector (1,WinNum) of selected mixtures
%                            path (i)=j, where j is the index of the selected mixture in time window i.

function path = calc_mix_Viterbi(logB,logA,GParams)
tmp = size(logB);    
WinNum = tmp(1);
SolNum = tmp(2);
   
% Initialize
delta(1,:)=logB(1,:);
 
% The loop
for t=2:WinNum
        [ delta(t,:)  psi(t-1,:) ] = ...
            max(  ( ones(SolNum,1)*delta(t-1,:) + GParams.ContinuityParam*logA{t-1}'  )' + ones(SolNum,1)*logB(t,:)  );
end
    
% Reconstruction of the states
[M,I] = max(delta(WinNum,:));
path(WinNum) = I;
for ti=WinNum-1:-1:1
    path(ti) = psi(ti,path(ti+1));    
end