%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Copyright (C) 2004 Adam Spiro and Aharon Bar-Hillel
%    An implementation of a spike sorting algorithm according to the paper:
%    "Spike Sorting: Bayesian Clustering of Non-Stationary Data" 
%    by Aharon Bar-Hillel, Adam Spiro and Eran Stark 
%    (http://www.cs.huji.ac.il/~aharonbh/papers/SpikeSort_NIPS04.ps)
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% this function gets a mixture model, and a data, and returns the
% assinment of points to models. points are assigned to models only if they belong 
% to it with confidence above Confidence. otherwise they are put in an additional
% 'unsure' class. 

function groups=confidentAss(Model, Data,EMParams,ConfidenceMin);

M=size(Model,1);
[ probabilities , ll ]=...
	calc_p_and_pll4(Data,Model,EMParams.single_cov_mat_flag,...
    [],1:length(Data),[],[],[],ones(1,length(Data)));

probabilities=(probabilities'./ (ones(M,1)*sum(probabilities')))';
if (size(probabilities,2)==1)
    maxProbs=probabilities';
    groups=ones(size(probabilities))';
else
    [ maxProbs, groups ]=max(probabilities');
end
groups(maxProbs<ConfidenceMin)=M+1;
