%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Copyright (C) 2004 Adam Spiro and Aharon Bar-Hillel
%    An implementation of a spike sorting algorithm according to the paper:
%    "Spike Sorting: Bayesian Clustering of Non-Stationary Data" 
%    by Aharon Bar-Hillel, Adam Spiro and Eran Stark 
%    (http://www.cs.huji.ac.il/~aharonbh/papers/SpikeSort_NIPS04.ps)
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% This function combines two gaussians and compute sevral quantities:
%        - the JS cost of merging two gausians
%       - the label gain achieved by the merging
%       - the dJS/dH ratio
%       - the merged model

% the gaussians g1, g2 are in the augmented format:
%       g{1} - the mean;
%       g{2} -  the covaraince
%       g{3} - the weight
%       g{4} - the determinant of g{2}
%       g{5} - the initial gaussians which are part of the gaussian.

% the function returns the difference in 'cost' and the merged Gaussian
% parameters in 'Merged' (in the same augmented format of g1, g2 )

function [ ratio, JS , H  ,  Merged ]=g_dist3(g1,g2);

% compute merged model
Merged{3}=g1{3}+g2{3};
p1=g1{3}/Merged{3};
p2=1-p1;
Merged{1}=p1*g1{1}+p2*g2{1};
Merged{2}= p1*g1{2}+p2*g2{2}+ p1*p2*(g1{1}-g2{1})*(g1{1}-g2{1})';
Merged{4}=det(Merged{2});
Merged{5}=union(g1{5},g2{5});

H=-p1*log(p1+eps)-p2*log(p2+eps);       % eps injected to avoid 0log()=Nan.
JS=0.5*(log(Merged{4})-p1*log(g1{4})-p2*log(g2{4}) );

H=Merged{3}*H;
JS=Merged{3}*JS;
ratio=JS/H;

