%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Copyright (C) 2004 Adam Spiro and Aharon Bar-Hillel
%    An implementation of a spike sorting algorithm according to the paper:
%    "Spike Sorting: Bayesian Clustering of Non-Stationary Data" 
%    by Aharon Bar-Hillel, Adam Spiro and Eran Stark 
%    (http://www.cs.huji.ac.il/~aharonbh/papers/SpikeSort_NIPS04.ps)
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% is=-1 if mirror without a single twin (should be background)
% is=n>0 if cluter n is the only negative cluster of the mirror cluster
% is=-2 if large scale
function is = isMirror(data,Labels,max_labels,C)
is=-2;

pos=find(data(:,1)>0);


Lind = find(max_labels==C);

ind=intersect(pos,Lind);
count=0;

count=zeros(1,max(Labels));
for i=ind
    f=findMirrorPoint(data,data(i,1),data(i,2),Labels,i);
    if (f>0)
        count(f)=count(f)+1;
    end
end

if (sum(count)/size(ind,2)>0.70) % how many spikes from cluster C have "mirror" points
    [a b]=max(count);
    if (a/sum(count)>0.70)
        is=b;
    else
        is=-1;
    end   
end

% f=0 if there's no mirror point
% f=n if the mirror point belongs to cluter n
function f = findMirrorPoint(data,x,y,Labels,i)
RADIUS=9000; % the radius in which to search for a mirror point

ind1=find(data(:,1)>-x-RADIUS);
ind2=find(data(:,1)<-x+RADIUS);
ind3=find(data(:,2)>y-RADIUS);
ind4=find(data(:,2)<y+RADIUS);

% ind1=find(data(max(1,i-1000):min(size(data,1),i+1000),1)>-x-RADIUS);
% ind2=find(data(max(1,i-1000):min(size(data,1),i+1000),1)<-x+RADIUS);
% ind3=find(data(max(1,i-1000):min(size(data,2),i+1000),2)>y-RADIUS);
% ind4=find(data(max(1,i-1000):min(size(data,2),i+1000),2)<y+RADIUS);

int=intersect(ind1,ind2);
int=intersect(int,ind3);
int=intersect(int,ind4);

if (isempty(int))
    f=0;
else
    [a b]=uniqFreq(Labels(int));
    [m1 m2]=max(b);
    f=a(m2);
end
    
