%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Copyright (C) 2004 Adam Spiro and Aharon Bar-Hillel
%    An implementation of a spike sorting algorithm according to the paper:
%    "Spike Sorting: Bayesian Clustering of Non-Stationary Data" 
%    by Aharon Bar-Hillel, Adam Spiro and Eran Stark 
%    (http://www.cs.huji.ac.il/~aharonbh/papers/SpikeSort_NIPS04.ps)
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%  mixingSolutions
%
%    This function calculates solutions based on the best solutions from 
%    past and future time windows, and transfers them to the current
%    window.

function [Solutions,LL] = mixingSolutions(PSolutions,PLL,data,time,GParams,N)

disp('Computing solutions based on past and future');

Solutions=cell(size(PSolutions,1),1);
LL=ones(size(PLL,1),0);

c=0;
for j=1:GParams.WinNum

    disp(sprintf('Time window %d',j));
    index=1;
    % create temporary data set to work on
    if GParams.UseConstWindowSizeFlag
        tData=data(c+1:c+N(j),:);
        c=c+N(j);
    else
        Inds=find(time>(j-1)*GParams.TimeWin & time<=j*GParams.TimeWin); % find the indexes of the i-th time window
        tData=data(Inds,:);
    end
    chunks=-ones(N(j),1);

    Offsets=setdiff(-GParams.TimeOffset:GParams.TimeOffset,0);
    for ofs=Offsets        % time offset
        if j+ofs>0 & j+ofs<=GParams.WinNum     % there is a relevant frame
            for m=1:GParams.TransferedSolNum  % Number of solutions to transfer
                if (m>size(PSolutions,2))
                    continue;
                end
                Transfered=PSolutions{j+ofs,m};
                if ~isempty(Transfered)
                    K = size(Transfered,1)-1;
                    
                    % direct transfer
                    Solutions{j,index}=Transfered;
                    [stam , logLikelihood ]=...
                        calc_map_labeling2(tData,Transfered);
                    LL(j,index) = logLikelihood;
                    index=index+1;
                    % transfer with small num of iterations
                    GParams.EMParams.param=Transfered;
                    GParams.EMParams.num_iterations=1;
                    [ Solutions{j,index}, tmpLL, Groups, z, stam ,exit_flags ]=...
                        mixture_of_gaussians5(tData, K, chunks, [],GParams.EMParams);
                    LL(j,index)=tmpLL(end);
                    index=index+1;
                    % transfer with large number of iterations
                    GParams.EMParams=rmfield(GParams.EMParams,'num_iterations');
                    [ Solutions{j,index}, tmpLL, Groups, z, stam ,exit_flags ]=...
                        mixture_of_gaussians5(tData, K, chunks, [],GParams.EMParams);
                    LL(j,index)=tmpLL(end);
                    index=index+1;
                end
            end     % of m loop
        end    % of if
    end % of ofs loop
end % of j loop

Solutions = [PSolutions'; Solutions']';
LL(find(LL==0))=-inf;
LL = [PLL' ; LL']';

Solutions=reorderBackGround(Solutions);