%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Copyright (C) 2004 Adam Spiro and Aharon Bar-Hillel
%    An implementation of a spike sorting algorithm according to the paper:
%    "Spike Sorting: Bayesian Clustering of Non-Stationary Data" 
%    by Aharon Bar-Hillel, Adam Spiro and Eran Stark 
%    (http://www.cs.huji.ac.il/~aharonbh/papers/SpikeSort_NIPS04.ps)
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Here we search for S&I clusters by first thinning (or filtering) the data and then using
% the EM. We then use our score to find the most seperable cluster.

function [max_score, max_labels, Sol, c]= seperateClust(data, Labels, GParams,repeat,xy)
max_score=0;
GParams.EMParams.BgrModelZeroMean=1;

% filter on the x or the y axis
if (xy==1)
    [a b]=unique(round(data(:,1)/100));
else
    [a b]=unique(round(data(:,2)/100));
end
[a sind]=sort(data(:,1));
[a sind2]=sort(data(:,2));
sind_inv(sind)=1:size(sind,1);

sdata=data(sind,:); % sorted by the x
sinv=sind_inv(sind2);
sdata2=sdata(sinv,:); % sorted by the y
sinv_inv(sinv)=1:size(sind,1);

% Repeat the searching until the most seperable clusters are found
for r=1:repeat
	cont=1;
    disp(sprintf('searching large scale/mirror candidates %d/%d',r,repeat));
	while cont
		[ Sol, tmpLL, Groups, z, stam ,exit_flags ]=mixture_of_gaussians5(data(b,:), 4, -ones(size(b,1),1), [],GParams.EMParams);
		t=Sol(end,:);
		Sol(end,:)=Sol(1,:);
		Sol(1,:)=t;
		labels=confidentAss(Sol, data,GParams.EMParams,0);
		s_labels=confidentAss(Sol, data(b,:),GParams.EMParams,0);
        bgsize=sum(s_labels==1)/size(s_labels,2);
        if (bgsize>0.05) 
            sprintf('Pruned, BG size: %d',bgsize);
            continue;
        else
            cont=0;
        end
		%visual_k_means2(data,labels);
        %score=calcSeperateScore(sdata,sdata2,labels(sind),sinv_inv)

        [score C]=calcSeperateScore_JS(Sol,[sum(GParams.SIW) sum(GParams.SIW)]);
        if max_score==0 % for initialization
            max_score=score;
            max_labels=labels;
        end
        if (score>max_score)
            max_score=score;
            max_labels=labels;
        end
	end
end

% Now that I have the mixture, I need to find out if there is a sepereted &
% a small cluster.
if (max_score<3.1)
    c=0;
    %disp('No large scale cluster found');
else
    c=0;
    %visual_k_means2(data,max_labels);
    for s=1:size(data,1)
        if (max_labels(s)>1 && Labels(s)==1)
            c=c+1;
        end
    end
    c;
end

