
% EMsetParams script

% this script handles the parameters setting of EM -mixture_of_gauusians5 and 6.
% the last two parameters (CW and SURatio ) only apply to mixture5.
% EMparams is a struct with the possible parameters (all have defaults)

% param                     :  a cell of kX3 start conditions : 
%                              the param(:,1) are centers . 
%                              the param(:,2) are cov matrixes. 
%                              the param(:,3) are the weights (scalars ) and they should sum to 1. 
%                              if param(1,3) is emtpy they are set to 1/k each
%                              and the initial conditions are serached for
%                              using multiple restarts.
%                              If chunklet exists and SCMF is 1 RCA is used
%                              in the initialization process.
%                            
% single_cov_mat_flag       : if 1 - a single covariance matrix is used .
%                               else - k matrixes. default 1
% ClassLabels               : if sent, the labels are used to asses the purity_acuracy score at every 
%                             iteration. the results are returned at z.
% late_oracle					 : early (0) / late (1) oracle flag. (default 1)
% reduce_arcs		    : controls pruning of anti chunklet arcs
%					0 - no pruning.
%					1 - prune contextually on every EM round
%					2 - prune once - only at the begining (default 2 )
% labeled                   : the same format as chunks, used for labels.
% fixed_covmat              : if 0 (default) will estimate cov matrix,
%                             if 1 will not estimate cov matrix and will use the one inputed.
%
% diagonal_covmat_flag      : if 0 will estimate full cov matrix. (default value)
%                             if 1 will estimate only diagonal elements of cov matrix
%
% num_iterations            : if set will run EM only fixed number of
%                             iterations given by its value. 
% 
% w                         : vector of weights of each data point.
%                           these weights are introduced as chunklets.
% aa_flag                   : when on, anti chunklet partition function
%                           aproximation is used. when off, anti are
%                           ignored in the PF calculation. The
%                           approximation doesn't seem to work, so set it
%                           to 0.
% CW                        : used when weights of the second kind are
%                           given. A list of weighted constraints in the
%                           format [ x y sign weight ].
% SURatio                   : used when weights of the second kind are
%                           given. sets the weight ratio between the
%                           supervised constraints group listed in CW and the
%                           group of unsupervised points (prior). the
%                           format is [ % supervised weight , %
%                           unsupervised weight ]
% ConstraintMapFlag         % when on, the assignment returned constrainedMap assignment
%                           % when off, generative model map is returned.
% number_of_starts          % number of initial conditions tried
%
% BgrModel -                when this option is ~=0, an additional backgruond model is
%                           added to the mixture, with covariance=cov(data)*BgrModel.
%                           This model mean and cov aren't updated during the EM.
%                           Only supported for mixture_of_gaussian6.
%
% BgrModelZeroMean -        If set to 1, the background model is centered at the 0.
%                           If set to 0, the background model is centered
%                           at the mean value of the data.
%
% MaxLLFlag - When 1, the LL output argument is the log likelihood of the single MAP
%                           assignment. When 0 (Default) the LL is the likelihood of the data, with 
%                           expectation over the hiddens.

% Notice : weights of the second kind ( weights over constraints ) overides
% the regular behaviour of the EM. When EMParams.CW is not  [] , anti and
% chunks are ignored.

s=size(data);
d=s(2);     % the dimension.
n=s(1);     % the number of samples.


if ~isfield(EMParams,'single_cov_mat_flag')
  single_cov_mat_flag=1;
  EMParams.single_cov_mat_flag=1;	
else
  single_cov_mat_flag=EMParams.single_cov_mat_flag;
end

dont_rand_covmat_flag=0;    % obsolete


global reduce_arcs;
if ~isfield(EMParams,'reduce_arcs')
   reduce_arcs=2;
   EMParams.reduce_arcs=2;
else 
   reduce_arcs=EMParams.reduce_arcs;
end


% organize constraints ( it is here beacause it has to come before the
% chooshing of start params ( c_inds, Nc_inds and singles are computed ).

global nets;	% the markov nets ( global used in calculate_partition_function2 and in calc_p_and_pll)
global OriginalNets;		% stores the full original nets ( before prunning ) in case 
if ~isfield(EMParams,'CW')     % regular scenario
    
    if ~isfield(EMParams,'w')
        EMParams.w=ones(1,length(data));
    end
    % divide the data into connected components and prepare graphs for each.
    global HiddensW; % used in calc_p_and_pll2 when resetting PairsW after edges prunning. 
    global SinglesW;    % used in PF computation
    global PairsW;      % used in PF computation.
    [ nets singles nc_inds c_inds oth SinglesW PairsW HiddensW] = ...
        organize_constraints_info2(chunks, anti_chunks , k ,EMParams );
    OriginalNets=nets;
else                        % weights of constraints
    UnsupervisedInds = setdiff(1:length(data),[ EMParams.CW(:,1) ; EMParams.CW(:,2) ]);
    c_inds=[];
    nc_inds=[];
    singles=[];
end

% background model - used in spike sorting application 

if ~isfield(EMParams,'BgrModel')     
    EMParams.BgrModel=0;
end

% initial conditions
%-------------------

if ~isfield(EMParams,'number_of_starts')
  EMParams.number_of_starts=30;	
end

if ~isfield(EMParams,'noiseParam')
    EMParams.noiseParam=[10 5 2];	 % alters the number of neighbors to be used for
    % choosing centers. when large, the number of  neighobrs used for
    % centroid computation is smaller.;	
    if EMParams.number_of_starts<5   % be darefull!
        noiseParam=[12 10 8];
    end
end

priorFlag=0;
ParamSF=0; % initial parameters search flag

if isfield(EMParams,'param')
    if size(EMParams.param,1)~= k+(EMParams.BgrModel~=0)    % initial condition number of models doesn't match declared k.
        EMParams=rmfield(EMParams,'param');      % cancel the given initial conditions
    end
end

if ~isfield(EMParams,'param') | isempty(EMParams.param)
    ParamSF=1;
    param=bestParams(data,k,chunks,EMParams,priorFlag,...
        c_inds,nc_inds,singles);
    
    % the previous start params setting :
    %EMParams.dont_rand_covmat_flag=0;
    %param=best_params_general(data,k,EMParams,chunks,number_of_starts);
    
    EMParams.param=param;
end

if EMParams.BgrModel
    k=k+1;
end

if ~isfield(EMParams,'ClassLabels')
    EMParams.ClassLabels=[];
end

global late_oracle;
if ~isfield(EMParams,'late_oracle')
  late_oracle=1;		% early or late oracle. default is late oracle.
  EMParams.late_oracle=1;
else
  late_oracle=EMParams.late_oracle;
end


% Noams regime :
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% init parameters for labeled points

if ~isfield(EMParams,'labeled')
    labeled=[];
    EMParams.labeled=[];
else
    labeled=EMParams.labeled;
end

if ~isempty(labeled)
    
  [lab_mat,colors,num_colors,labeled_points,index_labeled,param]=...
      label_for_mix_first(labeled,param);
  EMParams.lab_mat=lab_mat;
  EMParams.colors=colors;
  EMParams.num_colors=num_colors;
  
  if ParamSF &  num_colors>0 % initiate beta
    param=init_beta(param,colors,k,single_cov_mat_flag,data,labeled_points,index_labeled);
  end	
  
  % update all other paraeters
  % late_oracle=1 check this 
  anti_chunks=[];
  chunks=-ones(n,1);
  
else % no labels exist
  num_colors=0;
  labeled=-ones(n,1);
  lab_mat=[]; % data structure for labels
  labeled_points=[];
  index_labeled=[];
  colors=[];
  EMParams.labeled=-ones(n,1);
  EMParams.lab_mat=[]; % data structure for labels
  EMParams.labeled_points=[];
  EMParams.index_labeled=[];
  EMParams.colors=[];
  EMParams.num_colors=0;
end
% end labeled points
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ~isfield(EMParams,'fixed_covmat') % when fixed covmat is set , the comats aren't updated. param(:,2) is used without change. 
    fixed_covmat=0;
    EMParams.fixed_covmat=0;
else
    fixed_covmat=EMParams.fixed_covmat;
end
if fixed_covmat     % consistency
    dont_rand_covmat_flag=1;
    EMParams.dont_rand_covmat_flag=1;
end

if ~isfield(EMParams,'small_model_policy')
   small_model_policy=0;
   EMParams.small_model_policy=0;
else
    small_model_policy=EMParams.small_model_policy;
end

if ~isfield(EMParams,'diagonal_covmat_flag')
  diagonal_covmat_flag=0;
  EMParams.diagonal_covmat_flag=0;
else
    diagonal_covmat_flag=EMParams.diagonal_covmat_flag;
end

if ~isfield(EMParams,'num_iterations')
  num_iterations=0;
  EMParams.num_iterations=0;
else
    num_iterations=EMParams.num_iterations;
end

if ~isfield(EMParams,'w')
  w= [];
  EMParams.w=[];
else
    w=EMParams.w;
end

if ~isfield(EMParams,'CW')
  CW=[];    % indication that no weights of constraints are given
  EMParams.CW=[];
else
    CW=EMParams.CW;
end

if ~isfield(EMParams,'SURatio')
  SURatio=[1 0];   % no weight for unlabeled points  
  EMParams.SURatio=[1 0];
else
    SURatio=EMParams.SURatio;
end

global aa_flag;
if ~isfield(EMParams,'aa_flag') % the approximation currently is problematic
                                % don't use it. leave aa_flag=0 as default.
    aa_flag=0;
    EMParams.aa_flag=0;
else 
    aa_flag=EMParams.aa_flag;
end

if ~isfield(EMParams,'ConstrainedMapFlag') 
    EMParams.ConstrainedMapFlag=0;
end

z_record_flag=0;		% when set , a purity-acuracy score record is returned , in which
				% the purity-acuracy score is calculated in every iteration.
				% when 0 , only a final score is calculated -as [purity,accuracy,score].
global mm_flag;
mm_flag=0;				% choose between MM and EM (hard or sort k-means )   

stop_saf=1e-6;		% stopping criterion 
%small_model_policy=2;		% when 0 , small models are removed and no grace turn is given
									% when 1 , small models are freezed     
                           % when 2 , small models are removed and grace turn is given
% when multiple restarts is on, param are ignored.
% several starting conditions are tried and the one with the highest ll is used.
% the score for initial conditions takes into consideration the chunklett information, 
% but not the anti chunklet info.


% handle lack of chunkletts or anti-chunkletts
if ~exist('anti_chunks')
    anti_chunks=[];
end

if ~exist('chunks') | isempty(chunks)
   chunks=-ones(n,1);
end

% weigths of the second kind overide the regular chunks and anti 
if ~isempty(EMParams.CW)
    anti_chunks=[];
    chunks=-ones(n,1);
end

if ~isfield(EMParams,'MaxLLFlag');  
    EMPArams.MaxLLFlag=0;
end

