% this version is designed to support both kinds of EM weights.
% the weights are used in weighted RCA computation, in weighted random covmats
% computation and in seed choosing.

% data - the data in rows.
% k - number of models
% relevant fields of EMParams : Single_cov_mat_flag - how many covmats to
% produce. w, CW   - weights sampling . param - if param isn't empty but
% contains a matrix in (1,2)- it is used as a covmat ( relaces
% dont_rand_covmat_flag )

%returns

% param - the current randomly chosen params.

function [param] = ...
    select_start_params4(data,k,EMParams,noise_param,Chunklets,W,priorFlag) 

if ~exist('Chunklets')
    Chunklets=[];
end


% initialization
[n,d]=size(data);
used_flags=zeros(n,1); %will mark points used for centroids.
cumW=[ 0 ; cumsum(W) ];

% background model
ClassK=k;
if EMParams.BgrModel
    k=k+1;
    if (EMParams.BgrModelZeroMean) % If the bgr model is centered at 0.
        param{k,1}=zeros(size(mean(data)));
    else
        param{k,1}=mean(data);
    end
    param{k,2}=cov(data)*EMParams.BgrModel;
end
    
% the model weights are equiprobable
for i=1:k
    param{i,3}=1/k;  % uniform weights
end

% if a covmat is given from outside,  put it in the param
if isfield(EMParams,'param') & (~isempty(EMParams.param)) &(~isempty(EMParams.param{1,2}))
    param{1,2}=EMParams.param{1,2};
end


for j=1:ClassK
    % randomize cluster seed
    seed_ind=rand(1);
    seed_ind=find( diff( seed_ind<cumW ) ) ;  % weight reindex
    
    while used_flags(seed_ind)      % find unused seed
        seed_ind=rand(1);
        seed_ind=find( diff( seed_ind<cumW ) );  % weight reindex 
    end
    seed=data(seed_ind,:);  % select data point as seed.
    if isfield(EMParams,'param') & (~isempty(EMParams.param)) &(~isempty(EMParams.param{1,2}))
        % rand only centers
        
        %find neighbourhood  - using inpute covmat without limitation
        %on number of neighbors with respect to d. (see other case).
        dists= sum( (data-ones(n,1)*seed)'.*(inv(EMParams.param{1,2})*(data-ones(n,1)*seed)') );
        %dists= (data)*inv(param{1,2})*(-ones(n,1)*seed)';
        [ tmp inds ]=sort( dists );
        number_of_nei= ceil(n/(k*noise_param));
        inds=inds(1:number_of_nei);
        if ~isempty(Chunklets)         % chunklet closure
            ch=setdiff(unique(Chunklets(inds)),-1);
            inds=union(inds,find(ismember(Chunklets,ch)));
        end
        
        param{j,1}=W(inds)'*data(inds,:)/sum(W(inds));   
        
    else  % rand centers and cov matrices
        
        %find neighbourhood  - using euclidian metric.
        
        dists=sum(((data-ones(n,1)*seed).^2)');
        [ tmp inds ]=sort( dists );
        number_of_nei=max(round(n/(k*noise_param)),d+1);
        inds=inds(1:number_of_nei);
        
        if ~isempty(Chunklets)         % chunklet closure 
            % (assuming even mixture6 is given the chunklets argument ( not just pairs ))
            ch=setdiff(unique(Chunklets(inds)),-1);
            inds=union(inds,find(ismember(Chunklets,ch)));
        end
        
        % set params
        if isfield(EMParams,'diag_covmat_flag') & (EMParams.diag_covmat_flag)
            param{j,2}=cov(data(inds,:));
            param{j,2}=param{j,2}.*eye(size(param{j,2}));
        else
            param{j,2}=cov(data(inds,:));
        end
        
        %smooth cov mat:
        if priorFlag==1
            % use smoothing of covmat.
            T=eye(d);		% the prior covmat.
            alpha=n/20;		% the prior 'virtual sample' size.
            Sm=param{1,2}.*n;
            param{1,2}=(alpha*T+Sm)/(n+alpha);	% is this the map for a wishart distribution?
            % it needs to be determnied.
        end
        
        param{j,1}=W(inds)'*data(inds,:)/sum(W(inds)); 
    end
    
    used_flags(seed_ind)=1; % mark used point.
end

if ((isfield(EMParams,'param') & (~isempty(EMParams.param{1,2}))) & EMParams.single_cov_mat_flag )
    % unify the cov matrixes ( mean them ) and put the result in param{1,2}
    tmp=zeros(d,d);
    for j=1:k
        tmp=tmp + param{j,2};
    end
    param{1,2}=tmp/k;
    %clear all other cov mats!!!!!
    for j=2:k
        param{j,2}= [];
    end
end






