%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Copyright (C) 2004 Adam Spiro and Aharon Bar-Hillel
%    An implementation of a spike sorting algorithm according to the paper:
%    "Spike Sorting: Bayesian Clustering of Non-Stationary Data" 
%    by Aharon Bar-Hillel, Adam Spiro and Eran Stark 
%    (http://www.cs.huji.ac.il/~aharonbh/papers/SpikeSort_NIPS04.ps)
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%mainRun script

addpath Utils
addpath Algorithm

SetAllParams;

time = ones(size(data,1),1);

data=data(:,1:GParams.DataDim);
tag=data;

if GParams.FixedSeedFlag
    rand('state',GParams.FixedSeed);
    randn('state',GParams.FixedSeed);
end

%Gather solutions
disp('Gather Solutions');
tic
[ Solutions, LL,N ] = gatherSolutions3(data,time,GParams);
toc

Solutions=reorderBackGround(Solutions);

disp(sprintf('calc_mix_scores'));
tic
% Compute all the transition scores.
[ BiMixScores , Correspondences , Z] = calc_mix_scores(Solutions,N,GParams);
toc

disp(sprintf('calc_mix_Viterbi'));
tic
path = calc_mix_Viterbi(LL,BiMixScores,GParams);
toc

disp(sprintf('prepareUnitedLabels'));
tic
[ Labels,  RecordGraph ]=prepareUnitedLabels(data,Solutions,N,path,Correspondences,GParams);
toc

% compute simple path ( without between-time frames potentials )
[ SimpleLabels,  SimpleRecordGraph ]=prepareUnitedLabels(data,Solutions,N,ones(GParams.WinNum,1),Correspondences,GParams);

tic
%post processing (remove rare clusters)
[pLabels, pCorr, pBiMixScores,pRecordGraph, pSolutions,ppath,pLL]= ...
        post2(data,Solutions,N,path,Correspondences,BiMixScores,GParams,RecordGraph,Labels,LL);
toc

tic
% Remixing only with the path
[Ppath,PSolutions, PLL,PBiMixScores , PCorrespondences,PLabels,  PRecordGraph]=...
        RemixWithPath(pSolutions,pLL,ppath,data,time,GParams,N,tag,0);
toc

tic
% again, remove rare clusters
[pLabels2, pCorr2, pBiMixScores2,pRecordGraph2, pSolutions2,ppath2,pLL2]= ...
post2(data,PSolutions,N,Ppath,PCorrespondences,PBiMixScores,GParams,PRecordGraph,PLabels,PLL);
toc

tic
% A Second Remixing with the path
[ppath2,pSolutions2, pLL2,pBiMixScores2 , pCorr2,pLabels2,  pRecordGraph2]=...
        RemixWithPath(pSolutions2,pLL2,ppath2,data,time,GParams,N,tag,0);    
toc

tic
% Now detect the mirror and the large scale clusters
NewLabels = prepareLabelsWithMirrorLargeScale(data,GParams,pLabels2);
toc

tic
%Final arrangments
GetRidOfSmallClustersWithLargeCov
toc
